 /*
 *
 *           Copyright (c)  ClearSpeed Technology Plc 2006.
 *           All rights reserved.
 *
 * The following constitutes CONFIDENTIAL & PROPRIETARY SOURCE CODE of
 * ClearSpeed Technology Plc and the contents of this file may not be:
 * disclosed to any third party; copied; or duplicated, whether in whole
 * or in part, without the express written permission of a company Executive
 * Director of ClearSpeed Technology Plc.
 *
 */
/*!
  \file   cs_complex.h
  \brief  Clearspeed utilty header for crossplatform support of complex numbers

  \author MH
  \date   16-feb-2006
*/

#ifndef _CS_COMPLEX_H
#define _CS_COMPLEX_H

#ifdef _WIN32
# ifndef M_PI
#  define M_PI 3.14159265358979323846
# endif
# if (_MSC_VER < 1300)
#  define fabsf fabs
# endif
#endif


/* If <complex.h> is included, use the C99 complex type.  Otherwise
   define a type bit-compatible with C99 complex */
#ifdef _Complex_I
# include <complex.h>
# define CS_DEFINE_COMPLEX(R, C) typedef R _Complex C
# define CS_creal(C)  (creal(C))
# define CS_crealf(C) (crealf(C))
# define CS_creall(C) (creall(C))
# define CS_cimag(C)  (cimag(C))
# define CS_cimagf(C) (cimagf(C))
# define CS_cimagl(C) (cimagl(C))
# define CS_complex_assign(C,r,i) (C=(r)+I*(i))
#else
# define CS_DEFINE_COMPLEX(R, C) typedef struct {R re; R im;} C
# define CS_creal(C)  ((C).re)
# define CS_crealf(C) ((C).re)
# define CS_creall(C) ((C).re)
# define CS_cimag(C)  ((C).im)
# define CS_cimagf(C) ((C).im)
# define CS_cimagl(C) ((C).im)
# define CS_complex_assign(C,r,i) ((C).re=r,(C).im=i)
#endif


CS_DEFINE_COMPLEX(float , FloatComplex);
CS_DEFINE_COMPLEX(double, DoubleComplex);

#endif
