

/*
 * BEGIN COPYRIGHT MESSAGE
 *
 *	Copyright  (c)  ClearSpeed Technology plc 1997-2005.
 *	All rights reserved.
 *
 * The following constitutes CONFIDENTIAL & PROPRIETARY SOURCE CODE of
 * ClearSpeed Technology plc and the contents of this file may not be: disclosed to any
 * third party; copied; or duplicated, whether in whole or in part, without
 * the express written permission of a company Executive Director of
 * ClearSpeed Technology plc.
 *
 * END COPYRIGHT MESSAGE
 */


#ifndef __CSTHREADS_CSTHREAD_H__
#define __CSTHREADS_CSTHREAD_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32

#include <windows.h>
typedef LPTHREAD_START_ROUTINE CSThread_ThreadFunc;
typedef HANDLE CSThread_Thread;
typedef HANDLE CSThread_Semaphore;
typedef HANDLE CSThread_Mutex;

#define CSThread_Function DWORD WINAPI
#define CSThread_Return DWORD
#define CS_TIMEOUT WAIT_TIMEOUT

#else

#include <pthread.h>
typedef void* (*CSThread_ThreadFunc)(void*);
typedef pthread_t CSThread_Thread;
typedef pthread_cond_t CSThread_Semaphore;
typedef pthread_mutex_t CSThread_Mutex;

#define CSThread_Function void*
#define CSThread_Return void*
#define CS_TIMEOUT ETIMEDOUT

#endif

#define CS_NO_TIMEOUT ((unsigned int)(-1))

struct CSThread_CSMutex {
    CSThread_Mutex mut;
};

// This structure allows semaphores to support a "get current state" method,
// using the integer count. Under POSIX a mutex is required to use a semaphore,
// which is also used under Windows to change the count variable safely.
struct CSThread_CSSemaphore {
    CSThread_Semaphore sem;
    struct CSThread_CSMutex mut; // control access to this object
    unsigned int count;
};

// Thread functions
int csthread_newThread( const CSThread_ThreadFunc thread_func, const void* p_params, CSThread_Thread* thread );
int csthread_joinThread( const CSThread_Thread thread, int* exit_code );
void csthread_exitThread( const int exit_code );
int csthread_getThreadId( int* thread_id );

// Semaphore functions
int csthread_initSem( struct CSThread_CSSemaphore* sem, int initial_value );
int csthread_waitSem( struct CSThread_CSSemaphore* sem, unsigned int timeout_ms );
int csthread_sigSem( struct CSThread_CSSemaphore* sem, const unsigned int signal );
int csthread_getSem( struct CSThread_CSSemaphore* sem, int* value );
int csthread_destroySem( struct CSThread_CSSemaphore* sem );

// Mutex functions
int csthread_initMutex( struct CSThread_CSMutex* mut );
int csthread_lockMutex( struct CSThread_CSMutex* mut);
int csthread_unlockMutex( struct CSThread_CSMutex* mut);
int csthread_tryMutex( struct CSThread_CSMutex* mut, int* locked );
int csthread_destroyMutex( struct CSThread_CSMutex* mut);

#ifdef __cplusplus
}
#endif

#endif
